@@ -1,5 +1,5 @@
 /* Table mapping between kernel xtregset and GDB register cache.
-   Copyright (C) 2007-2017 Free Software Foundation, Inc.
+   Copyright (C) 2007-2018 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,22 +17,23 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
-typedef struct {
-  int   gdb_regnum;
-  int   gdb_offset;
-  int   ptrace_cp_offset;
-  int   ptrace_offset;
-  int   size;
-  int   coproc;
-  int   dbnum;
-  char* name
-;} xtensa_regtable_t;
+typedef struct
+{
+  int gdb_regnum;
+  int gdb_offset;
+  int ptrace_cp_offset;
+  int ptrace_offset;
+  int size;
+  int coproc;
+  int dbnum;
+  const char *name;
+} xtensa_regtable_t;
 
-#define XTENSA_ELF_XTREG_SIZE	4
+#define XTENSA_ELF_XTREG_SIZE 4
 
-const xtensa_regtable_t  xtensa_regmap_table[] = {
+const xtensa_regtable_t xtensa_regmap_table[] =
+{
   /* gnum,gofs,cpofs,ofs,siz,cp, dbnum,  name */
   {   44, 176,   0,   0,  4, -1, 0x020c, "scompare1" },
   { 0 }
 };
-
